@@ -4,7 +4,7 @@ HLCor <- function(formula,
                   distMatrix,uniqueGeo=NULL,adjMatrix,corrMatrix, covStruct=NULL,
                   verbose=c(trace=FALSE), 
                   control.dist=list(), ## provided by <corrfitme>_body if called through this function. Otherwise processed in not available and control.dist will be preprocessed.
-                  ...) { 
+                  ...) { # may contain processed
   spaMM.options(spaMM_glm_conv_crit=list(max=-Inf))
   time1 <- Sys.time()
   oricall <- match.call(expand.dots = TRUE) 
@@ -20,7 +20,7 @@ HLCor <- function(formula,
   # frst steps as in HLFit: (no need to test missing(data) in several functions)
   if (is.null(processed <- oricall$processed)) { ## no 'processed'
     ## FR->FR suggests we should add processed as argument of HLCor...
-    oricall$formula <- .stripFormula(formula)
+    oricall$formula <- .preprocess_formula(formula)
     #
     family <- .checkRespFam(family)
     if ( identical(family$family,"multi")) {
@@ -68,7 +68,8 @@ HLCor <- function(formula,
       preprocess.formal.args[["control.dist"]] <- control.dist ## because control.dist not in formals(HLfit)    #
       mc$processed <- do.call(.preprocess,preprocess.formal.args,envir=parent.frame(1L))
       # oricall$resid.model <- mc$processed$residModel
-      oricall$control.dist <- mc$processed$control.dist
+      # HLCor() DOES expect a DISTINCT control.dist argument in a call with a 'processed' argument
+      oricall$control.dist <- mc$processed$control_dist ## fix bug 26/12/2018 wrong extraction from processed.
       #mc$ranPars$ranCoefs <- NULL ## but new ranFix can be added by fitme/corrHLfit
       # HLCor_body() called below
     }
