@@ -57,7 +57,7 @@ if (spaMM.getOption("example_maxtime")>14) {
     fit1 <- HLfit(cbind(Dead,Alive) ~ (Trt + 0)/x + (x | Rep),
                   family=binomial(link="cloglog"),
                   data=X,HLmethod="ML",verbose=c(TRACE=FALSE)) ## 
-    testthat::expect_equal(logLik(fit1)[[1]],-1413.207,tolerance=1e-4)
+    testthat::expect_equal(logLik(fit1)[[1]],-1413.207,tolerance=1e-4) ## This is works in spprec too, with version 2.5.33
     if (FALSE) { ## not useful in routine tests
       if (file.exists((privtest <- "C:/home/francois/travail/stats/spaMMplus/spaMM/package/tests_other_pack/test-cloglog-lme4-glmmTMB.R"))) {
         source(privtest)
